﻿# This Source Code Form is subject to the
# terms of the Mozilla Public License, v.
# 2.0. If a copy of the MPL was not
# distributed with this file, You can
# obtain one at
# http://mozilla.org/MPL/2.0/.

bl_info = {
    "name": "MaSzyna T3D format",
    "author": "Grzegorz Firlejczyk",
    "version": (0, 1, 10),
    "blender": (2, 74, 0),
    "location": "File > Import-Export",
    "description": "Import-Export T3D models format for use in MaSzyna Train Simulator",
    "warning": "",
    "wiki_url": "",
    "category": "Import-Export"}

if "bpy" in locals():
    import importlib

import bpy
import bmesh
from bpy.props import (BoolProperty,
                       FloatProperty,
                       StringProperty,
                       EnumProperty,
                       )
from bpy_extras.io_utils import (ExportHelper,
                                 )
import os
import math

class ExportT3D(bpy.types.Operator, ExportHelper):
    """Save a Wavefront OBJ File"""

    bl_idname = "export_scene.t3d"
    bl_label = 'Export T3D'
    bl_options = {'PRESET'}
    #filepath = StringProperty(subtype='FILE_PATH')
    filename_ext = ".t3d"
    filter_glob = StringProperty(
            default="*.t3d;*.e3d",
            options={'HIDDEN'},
            )

    # context group
    use_selection = BoolProperty(
            name="Selection Only",
            description="Export selected objects only",
            default=False,
            )
    use_image_ext = BoolProperty(
            name="Image extension",
            description="Write with image extensions",
            default=False,
            )

    # object group
    use_mesh_modifiers = BoolProperty(
            name="Apply Modifiers",
            description="Apply modifiers (preview resolution)",
            default=True,
            )

    # extra data group
    #use_edges = BoolProperty(
    #        name="Include Edges",
    #        description="",
    #        default=True,
    #        )
    #use_smooth_groups = BoolProperty(
    #        name="Smooth Groups",
    #        description="Write sharp edges as smooth groups",
    #        default=False,
    #        )
    #use_smooth_groups_bitflags = BoolProperty(
    #        name="Bitflag Smooth Groups",
    #        description="Same as 'Smooth Groups', but generate smooth groups IDs as bitflags "
    #                    "(produces at most 32 different smooth groups, usually much less)",
    #        default=False,
    #        )
    #use_normals = BoolProperty(
    #        name="Write Normals",
    #        description="Export one normal per vertex and per face, to represent flat faces and sharp edges",
    #        default=True,
    #        )
    #use_uvs = BoolProperty(
    #        name="Include UVs",
    #        description="Write out the active UV coordinates",
    #        default=True,
    #        )
    #use_materials = BoolProperty(
    #        name="Write Materials",
    #        description="Write out the MTL file",
    #        default=True,
    #        )
    #use_triangles = BoolProperty(
    #        name="Triangulate Faces",
    #        description="Convert all faces to triangles",
    #        default=False,
    #        )
    #use_nurbs = BoolProperty(
    #        name="Write Nurbs",
    #        description="Write nurbs curves as OBJ nurbs rather than "
    #                    "converting to geometry",
    #        default=False,
    #        )
    #use_vertex_groups = BoolProperty(
    #        name="Polygroups",
    #        description="",
    #        default=False,
    #        )

    # grouping group
    #use_blen_objects = BoolProperty(
    #        name="Objects as OBJ Objects",
    #        description="",
    #        default=True,
    #        )
    #group_by_object = BoolProperty(
    #        name="Objects as OBJ Groups ",
    #        description="",
    #        default=False,
    #        )
    #group_by_material = BoolProperty(
    #        name="Material Groups",
    #        description="",
    #        default=False,
    #        )
    #keep_vertex_order = BoolProperty(
    #        name="Keep Vertex Order",
    #        description="",
    #        default=False,
    #        )

    #global_scale = FloatProperty(
    #        name="Scale",
    #        min=0.01, max=1000.0,
    #        default=1.0,
    #        )

    def execute(self, context):
        print("MaSzyna Exporter")
        
        self.filepath = bpy.path.ensure_ext(self.filepath, self.filename_ext)
        print(self.filepath)
        file = open(self.filepath,"w")
        scene = bpy.context.scene

        lend = "\n"
        list_of_objects = []
        if self.use_selection:
            list_of_objects = context.selected_objects
        else:
            list_of_objects = bpy.data.objects
        for obj in list_of_objects:
            print(obj.name) #print object name in console
            if obj.type == "MESH":
                print("\tMESH")
                print("\tHeader")
                file.writelines("//----------------" + lend)
                file.writelines("Parent: %s" % (obj.parent.name if obj.parent != None else "none")+ lend)
                file.writelines("Type: Mesh" + lend)
                file.writelines("Name: %s" % obj.name + lend)
                file.writelines("Anim: %s" % (obj["Anim"] if "Anim" in obj.keys() else "false")+lend)
                has_material = False
                has_texture = False
                if obj.active_material != None: #check if object has material pinned
                    has_material = True
                    if obj.active_material.active_texture != None \
                        and obj.active_material.active_texture.type == 'IMAGE' \
                        and obj.active_material.active_texture.image != None: #check if object has texture pinned
                        has_texture = True
                amb = obj.active_material.ambient if has_material else 0
                amb_list = (amb*255.0,amb*255.0,amb*255.0)
                empty_list = (0,0,0)
                file.writelines("Ambient: %.1f %.1f %.1f" % amb_list[:] + lend)
                file.writelines("Diffuse: %.1f %.1f %.1f" % ((obj.active_material.diffuse_color*255.0)[:] if has_material else empty_list[:]) + lend)
                file.writelines("Specular: %.1f %.1f %.1f" % ((obj.active_material.specular_color*255)[:] if has_material else empty_list[:]) + lend)
                si = "false" #Self illum parameter
                if has_material:
                    if obj.active_material.emit == 1: #if 1 then true
                        si = "true"
                    elif obj.active_material.emit > 0: #if not 1 and not 0 then value, if 0 then stays false
                        si = obj.active_material.emit * 100
                file.writelines("SelfIllum: %s" % si+lend)
                file.writelines("Wire: %s" % ("true" if obj.draw_type == "WIRE" else "false")+lend)
                file.writelines("WireSize: 1.0"+lend)
                file.writelines("Opacity: %.1f" % (obj.active_material.alpha * 100 if has_material and obj.active_material.use_transparency else 100)+lend)
                
                image_path = obj.active_material.active_texture.image.filepath_from_user().split("\\") if has_texture else ()
                path = ""
                save = False
                while(len(image_path)>1): #check if it is filename not a path
                    path += (image_path[0]+"/" if save else "")
                    if image_path[0] == "textures": #check if this is path start in maszyna folder
                        save = True
                    image_path = image_path[1:]
                path += ((os.path.splitext(image_path[0])[0] if not self.use_image_ext else image_path[0]).lower()) if has_texture else "none"
                file.writelines("Map: %s" % path + lend)
                file.writelines("MaxDistance: %i" % (obj["MaxDistance"] if "MaxDistance" in obj.keys() else 1000)+lend )
                file.writelines("MinDistance: %i" % (obj["MinDistance"] if "MinDistance" in obj.keys() else 0)+lend )
                file.writelines("Transform:"+lend)
                tm = obj.matrix_basis * obj.matrix_parent_inverse
                tm.transpose()
                print("\tMatrix")
                for line in tm:
                    file.writelines("\t%.6f %.6f %.6f %.6f" % line[:]+lend)
                #obj_t = obj.copy() #obiekt tymczasowy do triangulacji
                scene.objects.active = obj #aktywowanie obiektu dla potrzeb nastepnej linijki
                bpy.ops.object.modifier_add(type="TRIANGULATE") #dodanie triangulacji do obiektu
                #bpy.ops.object.modifier_apply(apply_as="DATA")
                mesh = obj.to_mesh(scene,self.use_mesh_modifiers,"PREVIEW")
                print("\tTriangles")
                file.writelines("NumVerts %i" % (len(mesh.polygons)*3)+lend)
                list_smooth_groups = mesh.calc_smooth_groups(True)
                file.flush()
                for face in mesh.polygons:
                    i = 0 #liczymy kazdy wierzcholek w kazdym face od 0 do 2
                    if face.use_smooth:
                        file.writelines(list_smooth_groups[0][face.index].__str__() + lend)
                    else:
                        file.writelines("0" + lend)
                    for vert_index in face.vertices:
                        tmp = list(mesh.vertices[vert_index].co.to_tuple())
                        if has_texture:
                            tmp.extend(mesh.uv_layers.active.data[face.loop_indices[i]].uv.to_tuple())
                        else:
                            tmp.extend((0,0))
                        file.writelines("%.6f %.6f %.6f %.6f %.6f \n" % tuple(tmp)[:])
                        i+=1
                    file.writelines(lend) #dodajemy odstep miedzy trojkatami
                obj.modifiers.remove(obj.modifiers[len(obj.modifiers)-1]) #usuwanie ostatniego obiektu modyfikacji
                file.flush()
                #END OF MESH
            elif obj.type == "LAMP" and bpy.data.lamps[obj.name].type == "SPOT": #only if lamp is spot
                lamp = bpy.data.lamps[obj.name] #get lamp object
                file.writelines("//----------------" + lend)
                file.writelines("Parent: %s" % (obj.parent.name if obj.parent != None else "none") + lend)
                file.writelines("Type: FreeSpotLight" + lend)
                file.writelines("Name: %s" % obj.name + lend)
                file.writelines("Anim: %s" % (obj["Anim"]+lend if "Anim" in obj.keys() else "false")+lend)
                file.writelines("Diffuse: %.1f %.1f %.1f" % (l.color.r,l.color.g,l.color.b)*255[:] + lend)
                file.writelines("SelfIllum: true"+lend)
                file.writelines("NearAttenStart: 0.0" + lend) #Blender hasn't this option
                file.writelines("NearAttenEnd: 40.0" + lend) #Blender hasn't this option
                file.writelines("UseNearAtten: false" + lend) #Blender hasn't this option so always false
                falloftype = 0
                if l.falloff_type == "INVERSE_LINEAR": #change string to integer used by 3ds max
                    falloftype = 1
                elif l.falloff_type == "INVERSE_SQUARE":
                    falloftype = 2
                file.writelines("FarAttenDecayType: %i" % falloftype + lend) #Type how dimming is calculated
                file.writelines("FarDecayRadius: %.1f" % l.distance + lend) #Max distance of light
                file.writelines("FalloffAngle: %.1f" % math.degrees(l.spot_size * (1+l.spot_blend)) + lend)
                file.writelines("HotspotAngle: %.1f" % math.degrees(l.spot_size) + lend)
                file.writelines("MaxDistance: %i" % (obj["MaxDistance"] if "MaxDistance" in obj.keys() else 1000)+lend )
                file.writelines("MinDistance: %i" % (obj["MinDistance"] if "MinDistance" in obj.keys() else 0)+lend )
                file.writelines("Transform:"+lend)
                tm = obj.matrix_basis * obj.matrix_parent_inverse
                tm.transpose()
                for line in tm:
                    file.writelines("\t%.6f %.6f %.6f %.6f" % line[:]+lend)
                file.flush()
                #END OF LAMP

        file.flush()
        file.close()


        print("Koniec")
        #from . import export_obj

        #from mathutils import Matrix
        #keywords = self.as_keywords(ignore=("axis_forward",
        #                                    "axis_up",
        #                                    "global_scale",
        #                                    "check_existing",
        #                                    "filter_glob",
        #                                    ))

        #global_matrix = (Matrix.Scale(self.global_scale, 4) *
        #                 axis_conversion(to_forward=self.axis_forward,
        #                                 to_up=self.axis_up,
        #                                 ).to_4x4())

        #keywords["global_matrix"] = global_matrix
        #return export_obj.save(self, context, **keywords)
        return {'FINISHED'}


#def menu_func_import(self, context):
#    self.layout.operator(ImportT3D.bl_idname, text="MaSzyna (.t3d)")


def menu_func_export(self, context):
    self.layout.operator(ExportT3D.bl_idname, text="MaSzyna (.t3d)")


def register():
    bpy.utils.register_module(__name__)

    #bpy.types.INFO_MT_file_import.append(menu_func_import)
    bpy.types.INFO_MT_file_export.append(menu_func_export)


def unregister():
    bpy.utils.unregister_module(__name__)

    #bpy.types.INFO_MT_file_import.remove(menu_func_import)
    bpy.types.INFO_MT_file_export.remove(menu_func_export)

if __name__ == "__main__":
    register()